/*
 * Copyright (c) 2009-2011 LodgON  (http://www.lodgon.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package com.lodgon.dali.core.ejb;

import com.lodgon.dali.core.entity.Application;
import com.lodgon.dali.core.entity.Content;
import com.lodgon.dali.core.entity.Group;
import com.lodgon.dali.core.entity.User;
import java.util.List;
import javax.ejb.EJB;
import javax.ejb.Stateless;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;
import org.eclipse.persistence.jpa.JpaEntityManager;
import org.eclipse.persistence.jpa.JpaHelper;

/**
 * This class provides the persistence operations for applications.
 *
 * @author erwin
 */
@Stateless
public class ApplicationBean {
  @EJB
  ContentBean contentBean;
  @EJB
  UserBean userBean;
  @EJB
  GroupBean groupBean;

  @PersistenceContext
  EntityManager em;

	/**
	 * Add an Application to persistent storage.
	 *
	 * @param application the application that has to be made persistent
	 * @return the persisted Application object.
	 */
  public Application create(Application application) {
    em.persist(application);
    return application;
  }

	/**
	 * Updates an Application in the persistent storage.
	 *
	 * @param application the application to update
	 * @return the updated Application object.
	 */
  public Application update(Application application) {
    return em.merge(application);
  }

	/**
	 * Remove the application and all the entities within that application that
	 * matches the specified application key. All Content, Group and User objects
	 * that reside within this application will be removed as well.
	 *
	 * @param appKey the key of the application to remove
	 */
  public void delete(String appKey) {
		if (appKey == null || "".equals(appKey)) {
			deleteContents("");
			deleteGroups("");
			deleteUsers("");
		} else {
		  Application a = em.find(Application.class, appKey);
	    if (a != null) {
	      em.remove(a);
				deleteContents(appKey);
				deleteGroups(appKey);
				deleteUsers(appKey);
			}
		}
  }

  private void deleteContents(String appKey) {
    Query q = em.createNamedQuery("Content.findByAppKey");
    q.setParameter("appKey", appKey);
    List<Content> contents = q.getResultList();
    for (Content content : contents) {
      contentBean.delete(content);
    }
  }

  private void deleteGroups(String appKey) {
    Query q = em.createNamedQuery("Group.findByAppKey");
    q.setParameter("appKey", appKey);
    List<Group> groups = q.getResultList();
    for (Group group : groups) {
      groupBean.delete(group);
    }
  }

  private void deleteUsers(String appKey) {
    Query q = em.createNamedQuery("User.findByAppKey");
    q.setParameter("appKey", appKey);
    List<User> users = q.getResultList();
    for (User user : users) {
      userBean.delete(user);
    }
  }

	/**
	 * Find an application specified by the application key.
	 *
	 * @param appKey the application key
	 * @return the Application that matches the application key
	 */
  public Application get(String appKey) {
    return em.find(Application.class, appKey);
  }

	/**
	 * Find a list of all the persisted application entities.
	 *
	 * @return a list of all applications in the persistence store
	 */
	public List<Application> findAll() {
		Query q = em.createNamedQuery("Application.findAll");
		return q.getResultList();
	}

	/**
   * Validates if an application mapped with <code>appKey</code> has the same
	 * secret key as the provided <code>secretKey</code>. If validation was
	 * successful, then this method will return <code>true</code>.
	 *
	 * @param appKey the key of the application to validate
	 * @param secretKey the secret to check against the application's secret key
	 * @return true when the application exists with the provided <code>appKey</code>
   *         and of which the secret key matches with the provided <code>secretKey</code>
	 */
	public boolean validateApplicationKey(String appKey, String secretKey) {
    Application app = em.find(Application.class, appKey);
    if (app != null) {
      return app.getSecretKey().equals(secretKey);
    }
    return false;
  }

  public void flush() {
    em.clear();
    JpaEntityManager jpaEm = JpaHelper.getEntityManager(em);
    jpaEm.getActiveSession().getIdentityMapAccessor().initializeAllIdentityMaps();
  }
}
