/*
 * Copyright (c) 2009-2011 LodgON  (http://www.lodgon.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package com.lodgon.dali.core.ejb;

import com.lodgon.dali.core.entity.Content;
import com.lodgon.dali.core.entity.ContentPermission;
import com.lodgon.dali.core.entity.Group;
import com.lodgon.dali.core.entity.GroupPermission;
import com.lodgon.dali.core.entity.TypePermission;
import com.lodgon.dali.core.entity.User;
import com.lodgon.dali.core.entity.UserPermission;
import java.util.List;
import javax.ejb.EJB;
import javax.ejb.Stateless;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;

/**
 * This class provides the persistence operations for permissions.
 *
 * @author erwin
 */
@Stateless
public class AuthorizationBean {
  @PersistenceContext
  EntityManager em;
	@EJB
	GroupBean groupBean;

	/**
	 * Grants a permission on a content to a user to perform an action.
	 *
	 * @param authorizableId the identifier of the user to grant the permission to
	 * @param targetId the identifier of the content to grant the permission on
	 * @param action the action being granted
	 * @throws DaliCoreException when the application key of the user conflicts
	 * with the application key of the content
	 */
	public void grantContentPermissionToUser(int authorizableId, int targetId, int action) throws DaliCoreException {
		grantContentPermissionToUser(em.find(User.class, authorizableId),
						em.find(Content.class, targetId), action);
	}

	/**
	 * Grants a permission on a content to a group to perform an action.
	 *
	 * @param authorizableId the identifier of the group to grant the permission to
	 * @param targetId the identifier of the content to grant the permission on
	 * @param action the action being granted
	 * @throws DaliCoreException when the application key of the group conflicts
	 * with the application key of the content
	 */
	public void grantContentPermissionToGroup(int authorizableId, int targetId, int action) throws DaliCoreException {
		grantContentPermissionToGroup(em.find(Group.class, authorizableId),
						em.find(Content.class, targetId), action);
	}

	/**
	 * Grants a permission on a managed Content entity to a managed User entity to
	 * perform an action.
	 *
	 * @param authorizable the managed User entity to grant the permission to
	 * @param target the managed Content entity to grant the permission on
	 * @param action the action being granted
	 * @throws DaliCoreException when the application key of the user conflicts
	 * with the application key of the content
	 */
	public void grantContentPermissionToUser(User authorizable, Content target, int action) throws DaliCoreException {
    if (! target.getAppKey().equals(authorizable.getAppKey())) {
      throw new DaliCoreException(DaliCoreException.APPKEY_CONFLICT, "appkey conflict " + authorizable.getAppKey());
    }
		ContentPermission contentPermission = getContentPermissionByUser(authorizable, target, action);
		if (contentPermission == null) {
			contentPermission = new ContentPermission();
			contentPermission.setUser(authorizable);
			contentPermission.setContent(target);
			contentPermission.setAction(action);
			em.persist(contentPermission);
		}
	}

	/**
	 * Grants a permission on a managed Content entity to a managed Group entity
	 * to perform an action.
	 *
	 * @param authorizable the managed Group entity to grant the permission to
	 * @param target the managed Content entity to grant the permission on
	 * @param action the action being granted
	 * @throws DaliCoreException when the application key of the group conflicts
	 * with the application key of the content
	 */
	public void grantContentPermissionToGroup(Group authorizable, Content target, int action) throws DaliCoreException {
    if (! target.getAppKey().equals(authorizable.getAppKey())) {
      throw new DaliCoreException(DaliCoreException.APPKEY_CONFLICT, "appkey conflict " + authorizable.getAppKey());
    }
		ContentPermission contentPermission = getContentPermissionByGroup(authorizable, target, action);
		if (contentPermission == null) {
			contentPermission = new ContentPermission();
			contentPermission.setGroup(authorizable);
			contentPermission.setContent(target);
			contentPermission.setAction(action);
			em.persist(contentPermission);
		}
	}

	/**
	 * Revokes a permission on a content from a user to perform an action. If no
	 * such permission exists, then this method does nothing.
	 *
	 * @param authorizableId the identifier of the user to revoke the permission from
	 * @param targetId the identifier of the content of which the permission is revoked
	 * @param action the action being revoked
	 * @throws DaliCoreException when the application key of the user conflicts
	 * with the application key of the content
	 */
	public void revokeContentPermissionFromUser(int authorizableId, int targetId, int action) throws DaliCoreException {
		revokeContentPermissionFromUser(em.find(User.class, authorizableId),
						em.find(Content.class, targetId), action);
	}

	/**
	 * Revokes a permission on a content from a group to perform an action. If no
	 * such permission exists, then this method does nothing.
	 *
	 * @param authorizableId the identifier of the group to revoke the permission from
	 * @param targetId the identifier of the content of which the permission is revoked
	 * @param action the action being revoked
	 * @throws DaliCoreException when the application key of the group conflicts
	 * with the application key of the content
	 */
	public void revokeContentPermissionFromGroup(int authorizableId, int targetId, int action) throws DaliCoreException {
		revokeContentPermissionFromGroup(em.find(Group.class, authorizableId),
						em.find(Content.class, targetId), action);
	}

	/**
	 * Revokes a permission on a managed Content entity from a managed User
	 * entity to perform an action. If no such permission exists, then this
	 * method does nothing.
	 *
	 * @param authorizable the managed User entity to revoke the permission from
	 * @param target the managed Content entity of which the permission is revoked
	 * @param action the action being revoked
	 * @throws DaliCoreException when the application key of the user conflicts
	 * with the application key of the content
	 */
	public void revokeContentPermissionFromUser(User authorizable, Content target, int action) throws DaliCoreException {
    if (! target.getAppKey().equals(authorizable.getAppKey())) {
      throw new DaliCoreException(DaliCoreException.APPKEY_CONFLICT, "appkey conflict " + authorizable.getAppKey());
    }
		ContentPermission contentPermission = getContentPermissionByUser(authorizable, target, action);
		if (contentPermission != null) {
			em.remove(contentPermission);
		}
	}

	/**
	 * Revokes a permission on a managed Content entity from a managed Group
	 * entity to perform an action. If no such permission exists, then this
	 * method does nothing.
	 *
	 * @param authorizable the managed Group entity to revoke the permission from
	 * @param target the managed Content entity of which the permission is revoked
	 * @param action the action being revoked
	 * @throws DaliCoreException when the application key of the group conflicts
	 * with the application key of the content
	 */
	public void revokeContentPermissionFromGroup(Group authorizable, Content target, int action) throws DaliCoreException {
    if (! target.getAppKey().equals(authorizable.getAppKey())) {
      throw new DaliCoreException(DaliCoreException.APPKEY_CONFLICT, "appkey conflict " + authorizable.getAppKey());
    }
		ContentPermission contentPermission = getContentPermissionByGroup(authorizable, target, action);
		if (contentPermission != null) {
			em.remove(contentPermission);
		}
	}

	/**
	 * Checks if the specified user has a content permission for the specified
	 * content and action.
	 *
	 * @param userId the identifier of the user
	 * @param contentId the identifier of the content
	 * @param action the action of the permission to check
	 * @return true when a permission was found for this user, false otherwise
	 */
  public boolean checkContentPermission(int userId, int contentId, int action) {
    return checkContentPermission(em.find(User.class, userId), em.find(Content.class, contentId), action);
  }

	/**
	 * Checks if the specified managed User entity has a content permission for
	 * the specified managed Content entity and action. A user has permission when
	 * a content permission for the specified content and action exists for the
	 * user or for one of the groups where the user is a member of.
	 *
	 * @param user the managed User entity
	 * @param content the managed Content entity
	 * @param action the action of the permission to check
	 * @return true when a content permission was found for this user, false otherwise
	 */
  public boolean checkContentPermission(User user, Content content, int action) {
    if (user == null) {
      return false;
    }

    if (getContentPermissionByUser(user, content, action) != null) {
      return true;
    }

		List<Group> groups = groupBean.getByMember(user.getAppKey(), user, null);
    for (Group group : groups) {
      if (getContentPermissionByGroup(group, content, action) != null) {
        return true;
      }
    }

    return false;
  }

	/**
	 * Find a content permission of a managed User entity for a specified managed
	 * Content entity and action.
	 *
	 * @param user the managed User entity
	 * @param content the managed Content entity
	 * @param action the action of the permission
	 * @return A content permission of the user for the specified content and action
	 */
  private ContentPermission getContentPermissionByUser(User user, Content content, int action) {
    Query query = em.createNamedQuery("ContentPermission.findByUserAndContentAndAction");
    query.setParameter("user", user);
    query.setParameter("content", content);
    query.setParameter("action", action);
    List<ContentPermission> contentPermissionsByUser = query.getResultList();
		return contentPermissionsByUser.isEmpty() ? null : contentPermissionsByUser.get(0);
  }

	/**
	 * Find a content permission of a managed Group entity for a specified managed
	 * Content entity and action.
	 *
	 * @param group the managed Group entity
	 * @param content the managed Content entity
	 * @param action the action of the permission
	 * @return A content permission of the group for the specified content and action
	 */
  private ContentPermission getContentPermissionByGroup(Group group, Content content, int action) {
    Query query = em.createNamedQuery("ContentPermission.findByGroupAndContentAndAction");
    query.setParameter("group", group);
    query.setParameter("content", content);
    query.setParameter("action", action);
    List<ContentPermission> contentPermissionsByGroup = query.getResultList();
		return contentPermissionsByGroup.isEmpty() ? null : contentPermissionsByGroup.get(0);
  }

	/**
	 * Find all content permissions for a content.
	 *
	 * @param contentId the identifier of the content
	 * @return All content permissions of the content
	 */
  public List<ContentPermission> getContentPermissionsByContent(int contentId) {
		return getContentPermissionsByContent(em.find(Content.class, contentId));
  }

	/**
	 * Find all content permissions for a managed Content entity.
	 *
	 * @param content the managed Content entity
	 * @return All content permissions of the content
	 */
  public List<ContentPermission> getContentPermissionsByContent(Content content) {
    Query q = em.createNamedQuery("ContentPermission.findByContent");
    q.setParameter("content", content);
    return q.getResultList();
  }

	/**
	 * Find all content permissions for a user.
	 *
	 * @param userId the identifier of the user
	 * @return All content permissions of the user
	 */
  public List<ContentPermission> getContentPermissionsByUser(int userId) {
		return getContentPermissionsByUser(em.find(User.class, userId));
  }

	/**
	 * Find all content permissions for a managed User entity.
	 *
	 * @param user the managed User entity
	 * @return All content permissions of the user
	 */
  public List<ContentPermission> getContentPermissionsByUser(User user) {
    Query q = em.createNamedQuery("ContentPermission.findByUser");
    q.setParameter("user", user);
    return q.getResultList();
  }

	/**
	 * Find all content permissions for a group.
	 *
	 * @param groupId the identifier of the group
	 * @return All content permissions of the group
	 */
  public List<ContentPermission> getContentPermissionsByGroup(int groupId) {
		return getContentPermissionsByGroup(em.find(Group.class, groupId));
  }

	/**
	 * Find all content permissions for a managed Group entity.
	 *
	 * @param group the managed Group entity
	 * @return All content permissions of the group
	 */
  public List<ContentPermission> getContentPermissionsByGroup(Group group) {
    Query q = em.createNamedQuery("ContentPermission.findByGroup");
    q.setParameter("group", group);
    return q.getResultList();
  }

	/**
	 * Find all content permissions of a user for a certain content.
	 *
	 * @param userId the identifier of the user
	 * @param contentId the identifier of the content
	 * @return All content permissions of the user for the content
	 */
  public List<ContentPermission> getContentPermissionsByUser(int userId, int contentId) {
    return getContentPermissionsByUser(em.find(User.class, userId), em.find(Content.class, contentId));
  }

	/**
	 * Find all content permissions of a managed User entity for a certain
	 * managed Content entity.
	 *
	 * @param user the managed User entity
	 * @param content the managed Content entity
	 * @return All content permissions of the user for the content
	 */
  public List<ContentPermission> getContentPermissionsByUser(User user, Content content) {
    Query q = em.createNamedQuery("ContentPermission.findByUserAndContent");
    q.setParameter("user", user);
    q.setParameter("content", content);
    return q.getResultList();
  }

	/**
	 * Find all content permissions of a group for a certain content.
	 *
	 * @param groupId the identifier of the user
	 * @param contentId the identifier of the content
	 * @return All content permissions of the group for the content
	 */
  public List<ContentPermission> getContentPermissionsByGroup(int groupId, int contentId) {
    return getContentPermissionsByGroup(em.find(Group.class, groupId), em.find(Content.class, contentId));
  }

	/**
	 * Find all content permissions of a managed Group entity for a certain
	 * managed Content entity.
	 *
	 * @param group the managed Group entity
	 * @param content the managed Content entity
	 * @return All content permissions of the group for the content
	 */
  public List<ContentPermission> getContentPermissionsByGroup(Group group, Content content) {
    Query q = em.createNamedQuery("ContentPermission.findByGroupAndContent");
    q.setParameter("group", group);
    q.setParameter("content", content);
    return q.getResultList();
  }

	/**
	 * Grants a permission on a custom type to a user to perform an action.
	 *
	 * @param authorizableId the identifier of the user to grant the permission to
	 * @param type the custom type to grant the permission on
	 * @param action the action being granted
	 */
	public void grantTypePermissionToUser(int authorizableId, int type, int action) {
		grantTypePermissionToUser(em.find(User.class, authorizableId),
						type, action);
	}

	/**
	 * Grants a permission on a custom type to a group to perform an action.
	 *
	 * @param authorizableId the identifier of the group to grant the permission to
	 * @param type the custom type to grant the permission on
	 * @param action the action being granted
	 */
	public void grantTypePermissionToGroup(int authorizableId, int type, int action) {
		grantTypePermissionToGroup(em.find(Group.class, authorizableId),
						type, action);
	}

	/**
	 * Grants a permission on a custom type to a managed User entity to perform an
	 * action.
	 *
	 * @param authorizable the managed User entity to grant the permission to
	 * @param type the custom type to grant the permission on
	 * @param action the action being granted
	 */
	public void grantTypePermissionToUser(User authorizable, int type, int action) {
		TypePermission typePermission = getTypePermissionByUser(authorizable, type, action);
		if (typePermission == null) {
			typePermission = new TypePermission();
			typePermission.setUser(authorizable);
			typePermission.setType(type);
			typePermission.setAction(action);
			em.persist(typePermission);
		}
	}

	/**
	 * Grants a permission on a custom type to a managed Group entity to perform an
	 * action.
	 *
	 * @param authorizable the managed Group entity to grant the permission to
	 * @param type the custom type to grant the permission on
	 * @param action the action being granted
	 */
	public void grantTypePermissionToGroup(Group authorizable, int type, int action) {
		TypePermission typePermission = getTypePermissionByGroup(authorizable, type, action);
		if (typePermission == null) {
			typePermission = new TypePermission();
			typePermission.setGroup(authorizable);
			typePermission.setType(type);
			typePermission.setAction(action);
			em.persist(typePermission);
		}
	}

	/**
	 * Revokes a permission on a custom type from a user to perform an action. If
	 * no such permission exists, then this method does nothing.
	 *
	 * @param authorizableId the identifier of the user to revoke the permission from
	 * @param type the custom type of which the permission is revoked
	 * @param action the action being revoked
	 */
	public void revokeTypePermissionFromUser(int authorizableId, int type, int action) {
		revokeTypePermissionFromUser(em.find(User.class, authorizableId),
						type, action);
	}

	/**
	 * Revokes a permission on a custom type from a group to perform an action. If
	 * no such permission exists, then this method does nothing.
	 *
	 * @param authorizableId the identifier of the group to revoke the permission from
	 * @param type the custom type of which the permission is revoked
	 * @param action the action being revoked
	 */
	public void revokeTypePermissionFromGroup(int authorizableId, int type, int action) {
		revokeTypePermissionFromGroup(em.find(Group.class, authorizableId),
						type, action);
	}

	/**
	 * Revokes a permission on a custom type from a managed User entity to perform
	 * an action. If no such permission exists, then this method does nothing.
	 *
	 * @param authorizable the managed User entity to revoke the permission from
	 * @param type the custom type of which the permission is revoked
	 * @param action the action being revoked
	 */
	public void revokeTypePermissionFromUser(User authorizable, int type, int action) {
		TypePermission typePermission = getTypePermissionByUser(authorizable, type, action);
		if (typePermission != null) {
			em.remove(typePermission);
		}
	}

	/**
	 * Revokes a permission on a custom type from a managed Group entity to
	 * perform an action. If no such permission exists, then this method does
	 * nothing.
	 *
	 * @param authorizable the managed Group entity to revoke the permission from
	 * @param type the custom type of which the permission is revoked
	 * @param action the action being revoked
	 */
	public void revokeTypePermissionFromGroup(Group authorizable, int type, int action) {
		TypePermission typePermission = getTypePermissionByGroup(authorizable, type, action);
		if (typePermission != null) {
			em.remove(typePermission);
		}
	}

	/**
	 * Checks if the specified user has a type permission of the specified type
	 * and action.
	 *
	 * @param userId the identifier of the user
	 * @param type the type of the permission to check
	 * @param action the action of the permission to check
	 * @return true when a permission was found for this user, false otherwise
	 */
  public boolean checkTypePermission(int userId, int type, int action) {
    return checkTypePermission(em.find(User.class, userId), type, action);
  }

	/**
	 * Checks if the specified managed User entity has a type permission of the
	 * specified type and action. A user has permission when a type permission of
	 * the specified type and action exists for the user or for one of the groups
	 * where the user is a member of.
	 *
	 * @param user the managed User entity
	 * @param type the type of the permission to check
	 * @param action the action of the permission to check
	 * @return true when a type permission was found for this user, false otherwise
	 */
  public boolean checkTypePermission(User user, int type, int action) {
    if (user == null) {
      return false;
    }

    if (getTypePermissionByUser(user, type, action) != null) {
      return true;
    }

		List<Group> groups = groupBean.getByMember(user.getAppKey(), user, null);
    for (Group group : groups) {
      if (getTypePermissionByGroup(group, type, action) != null) {
        return true;
      }
    }

    return false;
  }

	/**
	 * Find a type permission of a managed User entity that is of the specified
	 * type and action.
	 *
	 * @param user the managed User entity
	 * @param type the type of the permission
	 * @param action the action of the permission
	 * @return A type permission of the user that is of the specified type and action
	 */
  private TypePermission getTypePermissionByUser(User user, int type, int action) {
    Query query = em.createNamedQuery("TypePermission.findByUserAndTypeAndAction");
    query.setParameter("user", user);
    query.setParameter("type", type);
    query.setParameter("action", action);
    List<TypePermission> typePermissionsByUser = query.getResultList();
		return typePermissionsByUser.isEmpty() ? null : typePermissionsByUser.get(0);
  }

	/**
	 * Find a type permission of a managed Group entity that is of the specified
	 * type and action.
	 *
	 * @param group the managed Group entity
	 * @param type the type of the permission
	 * @param action the action of the permission
	 * @return A type permission of the user that is of the specified type and action
	 */
  private TypePermission getTypePermissionByGroup(Group group, int type, int action) {
    Query query = em.createNamedQuery("TypePermission.findByGroupAndTypeAndAction");
    query.setParameter("group", group);
    query.setParameter("type", type);
    query.setParameter("action", action);
    List<TypePermission> typePermissionsByGroup = query.getResultList();
		return typePermissionsByGroup.isEmpty() ? null : typePermissionsByGroup.get(0);
  }

	/**
	 * Find all type permissions for a certain type.
	 *
	 * @param type the type to search for
	 * @return All type permissions of the specified type
	 */
  public List<TypePermission> getTypePermissionsByType(int type) {
    Query q = em.createNamedQuery("TypePermission.findByType");
    q.setParameter("type", type);
    return q.getResultList();
  }

	/**
	 * Find all type permissions of a user.
	 *
	 * @param userId the identifier of the user
	 * @return All type permissions of the user
	 */
  public List<TypePermission> getTypePermissionsByUser(int userId) {
		return getTypePermissionsByUser(em.find(User.class, userId));
  }

	/**
	 * Find all type permissions of a managed User entity.
	 *
	 * @param user the managed User entity
	 * @return All type permissions of the user
	 */
  public List<TypePermission> getTypePermissionsByUser(User user) {
    Query q = em.createNamedQuery("TypePermission.findByUser");
    q.setParameter("user", user);
    return q.getResultList();
  }

	/**
	 * Find all type permissions of a group.
	 *
	 * @param groupId the identifier of the group
	 * @return All type permissions of the group
	 */
  public List<TypePermission> getTypePermissionsByGroup(int groupId) {
		return getTypePermissionsByGroup(em.find(Group.class, groupId));
  }

	/**
	 * Find all type permissions of a managed Group entity.
	 *
	 * @param group the managed Group entity
	 * @return All type permissions of the group
	 */
  public List<TypePermission> getTypePermissionsByGroup(Group group) {
    Query q = em.createNamedQuery("TypePermission.findByGroup");
    q.setParameter("group", group);
    return q.getResultList();
  }

	/**
	 * Find all type permissions of a user that are of the specified type.
	 *
	 * @param userId the identifier of the user
	 * @param type the type to search for
	 * @return All type permissions of the user that are of the specified type
	 */
  public List<TypePermission> getTypePermissionsByUser(int userId, int type) {
    return getTypePermissionsByUser(em.find(User.class, userId), type);
  }

	/**
	 * Find all type permissions of a managed User entity that are of the
	 * specified type.
	 *
	 * @param user the managed User entity
	 * @param type the type to search for
	 * @return All type permissions of the user that are of the specified type
	 */
  public List<TypePermission> getTypePermissionsByUser(User user, int type) {
    Query q = em.createNamedQuery("TypePermission.findByUserAndType");
    q.setParameter("user", user);
    q.setParameter("type", type);
    return q.getResultList();
  }

	/**
	 * Find all type permissions of a group that are of the specified type.
	 *
	 * @param groupId the identifier of the group
	 * @param type the type to search for
	 * @return All type permissions of the group that are of the specified type
	 */
  public List<TypePermission> getTypePermissionsByGroup(int groupId, int type) {
		return getTypePermissionsByGroup(em.find(Group.class, groupId), type);
  }

	/**
	 * Find all type permissions of a managed Group entity that are of the
	 * specified type.
	 *
	 * @param group the managed Group entity
	 * @param type the type to search for
	 * @return All type permissions of the group that are of the specified type
	 */
  public List<TypePermission> getTypePermissionsByGroup(Group group, int type) {
    Query q = em.createNamedQuery("TypePermission.findByGroupAndType");
    q.setParameter("group", group);
    q.setParameter("type", type);
    return q.getResultList();
  }

	/**
	 * Grants a permission on a target group to a user to perform an action.
	 *
	 * @param authorizableId the identifier of the user to grant the permission to
	 * @param targetId the identifier of the target group to grant the permission on
	 * @param action the action being granted
	 * @throws DaliCoreException when the application key of the user conflicts
	 * with the application key of the target group
	 */
	public void grantGroupPermissionToUser(int authorizableId, int targetId, int action) throws DaliCoreException {
		grantGroupPermissionToUser(em.find(User.class, authorizableId),
						em.find(Group.class, targetId), action);
	}

	/**
	 * Grants a permission on a target group to a group to perform an action.
	 *
	 * @param authorizableId the identifier of the group to grant the permission to
	 * @param targetId the identifier of the target group to grant the permission on
	 * @param action the action being granted
	 * @throws DaliCoreException when the application key of the group conflicts
	 * with the application key of the target group
	 */
	public void grantGroupPermissionToGroup(int authorizableId, int targetId, int action) throws DaliCoreException {
		grantGroupPermissionToGroup(em.find(Group.class, authorizableId),
						em.find(Group.class, targetId), action);
	}

	/**
	 * Grants a permission on a managed target Group entity to a managed User
	 * entity to perform an action.
	 *
	 * @param authorizable the managed User entity to grant the permission to
	 * @param target the managed target Group entity to grant the permission on
	 * @param action the action being granted
	 * @throws DaliCoreException when the application key of the user conflicts
	 * with the application key of the target group
	 */
	public void grantGroupPermissionToUser(User authorizable, Group target, int action) throws DaliCoreException {
    if (! target.getAppKey().equals(authorizable.getAppKey())) {
      throw new DaliCoreException(DaliCoreException.APPKEY_CONFLICT, "appkey conflict " + authorizable.getAppKey());
    }
		GroupPermission groupPermission = getGroupPermissionByUser(authorizable, target, action);
		if (groupPermission == null) {
			groupPermission = new GroupPermission();
			groupPermission.setUser(authorizable);
			groupPermission.setTarget(target);
			groupPermission.setAction(action);
			em.persist(groupPermission);
		}
	}

	/**
	 * Grants a permission on a managed target Group entity to a managed Group
	 * entity to perform an action.
	 *
	 * @param authorizable the managed Group entity to grant the permission to
	 * @param target the managed target Group entity to grant the permission on
	 * @param action the action being granted
	 * @throws DaliCoreException when the application key of the group conflicts
	 * with the application key of the target group
	 */
	public void grantGroupPermissionToGroup(Group authorizable, Group target, int action) throws DaliCoreException {
    if (! target.getAppKey().equals(authorizable.getAppKey())) {
      throw new DaliCoreException(DaliCoreException.APPKEY_CONFLICT, "appkey conflict " + authorizable.getAppKey());
    }
		GroupPermission groupPermission = getGroupPermissionByGroup(authorizable, target, action);
		if (groupPermission == null) {
			groupPermission = new GroupPermission();
			groupPermission.setGroup(authorizable);
			groupPermission.setTarget(target);
			groupPermission.setAction(action);
			em.persist(groupPermission);
		}
	}

	/**
	 * Revokes a permission on a target group from a user to perform an action. If
	 * no such permission exists, then this method does nothing.
	 *
	 * @param authorizableId the identifier of the user to revoke the permission from
	 * @param targetId the identifier of the target group of which the permission is revoked
	 * @param action the action being revoked
	 * @throws DaliCoreException when the application key of the user conflicts
	 * with the application key of the target group
	 */
	public void revokeGroupPermissionFromUser(int authorizableId, int targetId, int action) throws DaliCoreException {
		revokeGroupPermissionFromUser(em.find(User.class, authorizableId),
						em.find(Group.class, targetId), action);
	}

	/**
	 * Revokes a permission on a target group from a group to perform an action.
	 * If no such permission exists, then this method does nothing.
	 *
	 * @param authorizableId the identifier of the group to revoke the permission from
	 * @param targetId the identifier of the target group of which the permission is revoked
	 * @param action the action being revoked
	 * @throws DaliCoreException when the application key of the group conflicts
	 * with the application key of the target group
	 */
	public void revokeGroupPermissionFromGroup(int authorizableId, int targetId, int action) throws DaliCoreException {
		revokeGroupPermissionFromGroup(em.find(Group.class, authorizableId),
						em.find(Group.class, targetId), action);
	}

	/**
	 * Revokes a permission on a managed target Group entity from a managed User
	 * entity to perform an action. If no such permission exists, then this method
	 * does nothing.
	 *
	 * @param authorizable the managed User entity to revoke the permission from
	 * @param target the managed target Group entity of which the permission is revoked
	 * @param action the action being revoked
	 * @throws DaliCoreException when the application key of the user conflicts
	 * with the application key of the target group
	 */
	public void revokeGroupPermissionFromUser(User authorizable, Group target, int action) throws DaliCoreException {
    if (! target.getAppKey().equals(authorizable.getAppKey())) {
      throw new DaliCoreException(DaliCoreException.APPKEY_CONFLICT, "appkey conflict " + authorizable.getAppKey());
    }
		GroupPermission groupPermission = getGroupPermissionByUser(authorizable, target, action);
		if (groupPermission != null) {
			em.remove(groupPermission);
		}
	}

	/**
	 * Revokes a permission on a managed target Group entity from a managed Group
	 * entity to perform an action. If no such permission exists, then this method
	 * does nothing.
	 *
	 * @param authorizable the managed Group entity to revoke the permission from
	 * @param target the managed target Group entity of which the permission is revoked
	 * @param action the action being revoked
	 * @throws DaliCoreException when the application key of the group conflicts
	 * with the application key of the target group
	 */
	public void revokeGroupPermissionFromGroup(Group authorizable, Group target, int action) throws DaliCoreException {
    if (! target.getAppKey().equals(authorizable.getAppKey())) {
      throw new DaliCoreException(DaliCoreException.APPKEY_CONFLICT, "appkey conflict " + authorizable.getAppKey());
    }
		GroupPermission groupPermission = getGroupPermissionByGroup(authorizable, target, action);
		if (groupPermission != null) {
			em.remove(groupPermission);
		}
	}

	/**
	 * Checks if the specified user has a group permission for the specified
	 * group and action.
	 *
	 * @param userId the identifier of the user
	 * @param groupId the identifier of the group
	 * @param action the action of the permission to check
	 * @return true when a permission was found for this user, false otherwise
	 */
  public boolean checkGroupPermission(int userId, int groupId, int action) {
		return checkGroupPermission(em.find(User.class, userId), em.find(Group.class, groupId), action);
	}

	/**
	 * Checks if the specified managed User entity has a group permission for the
	 * specified managed Group entity and action. A user has permission when a
	 * group permission for the specified group and action exists for the user or
	 * for one of the groups where the user is a member of.
	 *
	 * @param user the managed User entity
	 * @param group the managed Group entity
	 * @param action the action of the permission to check
	 * @return true when a group permission was found for this user, false otherwise
	 */
  public boolean checkGroupPermission(User user, Group group, int action) {
    if (user == null) {
      return false;
    }

    if (getGroupPermissionByUser(user, group, action) != null) {
      return true;
    }

		List<Group> groups = groupBean.getByMember(user.getAppKey(), user, null);
    for (Group g : groups) {
      if (getGroupPermissionByGroup(g, group, action) != null) {
        return true;
      }
    }

    return false;
  }

	/**
	 * Find a group permission of a managed User entity for a specified managed
	 * target Group entity and action.
	 *
	 * @param user the managed User entity
	 * @param targetGroup the managed target Group entity
	 * @param action the action of the permission
	 * @return A group permission of the user for the specified target group and action
	 */
  private GroupPermission getGroupPermissionByUser(User user, Group targetGroup, int action) {
    Query query = em.createNamedQuery("GroupPermission.findByUserAndTargetAndAction");
    query.setParameter("user", user);
    query.setParameter("target", targetGroup);
    query.setParameter("action", action);
    List<GroupPermission> groupPermissionsByUser = query.getResultList();
		return groupPermissionsByUser.isEmpty() ? null : groupPermissionsByUser.get(0);
  }

	/**
	 * Find a group permission of a managed Group entity for a specified managed
	 * target Group entity and action.
	 *
	 * @param group the managed Group entity
	 * @param targetGroup the managed target Group entity
	 * @param action the action of the permission
	 * @return A group permission of the group for the specified target group and action
	 */
  private GroupPermission getGroupPermissionByGroup(Group group, Group targetGroup, int action) {
    Query query = em.createNamedQuery("GroupPermission.findByGroupAndTargetAndAction");
    query.setParameter("group", group);
    query.setParameter("target", targetGroup);
    query.setParameter("action", action);
    List<GroupPermission> groupPermissionsByGroup = query.getResultList();
		return groupPermissionsByGroup.isEmpty() ? null : groupPermissionsByGroup.get(0);
  }

	/**
	 * Find all group permissions of a target group.
	 *
	 * @param targetGroupId the identifier of the target group
	 * @return All group permissions of the target group
	 */
  public List<GroupPermission> getGroupPermissionsByTarget(int targetGroupId) {
		return getGroupPermissionsByTarget(em.find(Group.class, targetGroupId));
  }

	/**
	 * Find all group permissions of a managed target Group entity.
	 *
	 * @param targetGroup the managed target Group entity
	 * @return All group permissions of the target group
	 */
  public List<GroupPermission> getGroupPermissionsByTarget(Group targetGroup) {
    Query q = em.createNamedQuery("GroupPermission.findByTarget");
    q.setParameter("target", targetGroup);
    return q.getResultList();
  }

	/**
	 * Find all group permissions of a user.
	 *
	 * @param userId the identifier of the user
	 * @return All group permissions of the user
	 */
  public List<GroupPermission> getGroupPermissionsByUser(int userId) {
		return getGroupPermissionsByUser(em.find(User.class, userId));
  }

	/**
	 * Find all group permissions of a managed User entity.
	 *
	 * @param user the managed User entity
	 * @return All group permissions of the user
	 */
  public List<GroupPermission> getGroupPermissionsByUser(User user) {
    Query q = em.createNamedQuery("GroupPermission.findByUser");
    q.setParameter("user", user);
    return q.getResultList();
  }

	/**
	 * Find all group permissions of a group.
	 *
	 * @param groupId the identifier of the group
	 * @return All group permissions of the group
	 */
  public List<GroupPermission> getGroupPermissionsByGroup(int groupId) {
		return getGroupPermissionsByGroup(em.find(Group.class, groupId));
  }

	/**
	 * Find all group permissions of a managed Group entity.
	 *
	 * @param group the managed Group entity
	 * @return All group permissions of the group
	 */
  public List<GroupPermission> getGroupPermissionsByGroup(Group group) {
    Query q = em.createNamedQuery("GroupPermission.findByGroup");
    q.setParameter("group", group);
    return q.getResultList();
  }

	/**
	 * Find all group permissions of a user for a certain target group.
	 *
	 * @param userId the identifier of the user
	 * @param targetGroupId the identifier of the target group
	 * @return All group permissions of the user for the target group
	 */
  public List<GroupPermission> getGroupPermissionsByUser(int userId, int targetGroupId) {
    return getGroupPermissionsByUser(em.find(User.class, userId), em.find(Group.class, targetGroupId));
  }

	/**
	 * Find all group permissions of a managed User entity for a certain managed
	 * target Group entity.
	 *
	 * @param user the managed User entity
	 * @param targetGroup the managed target Group entity
	 * @return All group permissions of the user for the target group
	 */
  public List<GroupPermission> getGroupPermissionsByUser(User user, Group targetGroup) {
    Query q = em.createNamedQuery("GroupPermission.findByUserAndTarget");
    q.setParameter("user", user);
    q.setParameter("target", targetGroup);
    return q.getResultList();
  }

	/**
	 * Find all group permissions of a group for a certain target group.
	 *
	 * @param groupId the identifier of the group
	 * @param targetGroupId the identifier of the target group
	 * @return All group permissions of the group for the target group
	 */
  public List<GroupPermission> getGroupPermissionsByGroup(int groupId, int targetGroupId) {
    return getGroupPermissionsByGroup(em.find(Group.class, groupId), em.find(Group.class, targetGroupId));
  }

	/**
	 * Find all group permissions of a managed Group entity for a certain managed
	 * target Group entity.
	 *
	 * @param group the managed Group entity
	 * @param targetGroup the managed target Group entity
	 * @return All group permissions of the group for the target group
	 */
  public List<GroupPermission> getGroupPermissionsByGroup(Group group, Group targetGroup) {
    Query q = em.createNamedQuery("GroupPermission.findByGroupAndTarget");
    q.setParameter("group", group);
    q.setParameter("target", targetGroup);
    return q.getResultList();
  }

	/**
	 * Grants a permission on a target user to a user to perform an action.
	 *
	 * @param authorizableId the identifier of the user to grant the permission to
	 * @param targetId the identifier of the target user to grant the permission on
	 * @param action the action being granted
	 * @throws DaliCoreException when the application key of the user conflicts
	 * with the application key of the target user
	 */
	public void grantUserPermissionToUser(int authorizableId, int targetId, int action) throws DaliCoreException {
		grantUserPermissionToUser(em.find(User.class, authorizableId),
						em.find(User.class, targetId), action);
	}

	/**
	 * Grants a permission on a target user to a group to perform an action.
	 *
	 * @param authorizableId the identifier of the group to grant the permission to
	 * @param targetId the identifier of the target user to grant the permission on
	 * @param action the action being granted
	 * @throws DaliCoreException when the application key of the group conflicts
	 * with the application key of the target user
	 */
	public void grantUserPermissionToGroup(int authorizableId, int targetId, int action) throws DaliCoreException {
		grantUserPermissionToGroup(em.find(Group.class, authorizableId),
						em.find(User.class, targetId), action);
	}

	/**
	 * Grants a permission on a managed target User entity to a managed User
	 * entity to perform an action.
	 *
	 * @param authorizable the managed User entity to grant the permission to
	 * @param target the managed target User entity to grant the permission on
	 * @param action the action being granted
	 * @throws DaliCoreException when the application key of the user conflicts
	 * with the application key of the target user
	 */
	public void grantUserPermissionToUser(User authorizable, User target, int action) throws DaliCoreException {
    if (! target.getAppKey().equals(authorizable.getAppKey())) {
      throw new DaliCoreException(DaliCoreException.APPKEY_CONFLICT, "appkey conflict " + authorizable.getAppKey());
    }
		UserPermission userPermission = getUserPermissionByUser(authorizable, target, action);
		if (userPermission == null) {
			userPermission = new UserPermission();
			userPermission.setUser(authorizable);
			userPermission.setTarget(target);
			userPermission.setAction(action);
			em.persist(userPermission);
		}
	}

	/**
	 * Grants a permission on a managed target User entity to a managed Group
	 * entity to perform an action.
	 *
	 * @param authorizable the managed Group entity to grant the permission to
	 * @param target the managed target User entity to grant the permission on
	 * @param action the action being granted
	 * @throws DaliCoreException when the application key of the group conflicts
	 * with the application key of the target user
	 */
	public void grantUserPermissionToGroup(Group authorizable, User target, int action) throws DaliCoreException {
    if (! target.getAppKey().equals(authorizable.getAppKey())) {
      throw new DaliCoreException(DaliCoreException.APPKEY_CONFLICT, "appkey conflict " + authorizable.getAppKey());
    }
		UserPermission userPermission = getUserPermissionByGroup(authorizable, target, action);
		if (userPermission == null) {
			userPermission = new UserPermission();
			userPermission.setGroup(authorizable);
			userPermission.setTarget(target);
			userPermission.setAction(action);
			em.persist(userPermission);
		}
	}

	/**
	 * Revokes a permission on a target user from a user to perform an action. If
	 * no such permission exists, then this method does nothing.
	 *
	 * @param authorizableId the identifier of the user to revoke the permission from
	 * @param targetId the identifier of the target user of which the permission is revoked
	 * @param action the action being revoked
	 * @throws DaliCoreException when the application key of the user conflicts
	 * with the application key of the target user
	 */
	public void revokeUserPermissionFromUser(int authorizableId, int targetId, int action) throws DaliCoreException {
		revokeUserPermissionFromUser(em.find(User.class, authorizableId),
						em.find(User.class, targetId), action);
	}

	/**
	 * Revokes a permission on a target user from a group to perform an action. If
	 * no such permission exists, then this method does nothing.
	 *
	 * @param authorizableId the identifier of the group to revoke the permission from
	 * @param targetId the identifier of the target user of which the permission is revoked
	 * @param action the action being revoked
	 * @throws DaliCoreException when the application key of the group conflicts
	 * with the application key of the target user
	 */
	public void revokeUserPermissionFromGroup(int authorizableId, int targetId, int action) throws DaliCoreException {
		revokeUserPermissionFromGroup(em.find(Group.class, authorizableId),
						em.find(User.class, targetId), action);
	}

	/**
	 * Revokes a permission on a managed target User entity from a managed User
	 * entity to perform an action. If no such permission exists, then this method
	 * does nothing.
	 *
	 * @param authorizable the managed User entity to revoke the permission from
	 * @param target the managed target User entity of which the permission is revoked
	 * @param action the action being revoked
	 * @throws DaliCoreException when the application key of the user conflicts
	 * with the application key of the target user
	 */
	public void revokeUserPermissionFromUser(User authorizable, User target, int action) throws DaliCoreException {
    if (! target.getAppKey().equals(authorizable.getAppKey())) {
      throw new DaliCoreException(DaliCoreException.APPKEY_CONFLICT, "appkey conflict " + authorizable.getAppKey());
    }
		UserPermission userPermission = getUserPermissionByUser(authorizable, target, action);
		if (userPermission != null) {
			em.remove(userPermission);
		}
	}

	/**
	 * Revokes a permission on a managed target User entity from a managed Group
	 * entity to perform an action. If no such permission exists, then this method
	 * does nothing.
	 *
	 * @param authorizable the managed Group entity to revoke the permission from
	 * @param target the managed target User entity of which the permission is revoked
	 * @param action the action being revoked
	 * @throws DaliCoreException when the application key of the group conflicts
	 * with the application key of the target user
	 */
	public void revokeUserPermissionFromGroup(Group authorizable, User target, int action) throws DaliCoreException {
    if (! target.getAppKey().equals(authorizable.getAppKey())) {
      throw new DaliCoreException(DaliCoreException.APPKEY_CONFLICT, "appkey conflict " + authorizable.getAppKey());
    }
		UserPermission userPermission = getUserPermissionByGroup(authorizable, target, action);
		if (userPermission != null) {
			em.remove(userPermission);
		}
	}

	/**
	 * Checks if the specified user has a user permission for the specified
	 * target user and action.
	 *
	 * @param userId the identifier of the user
	 * @param targetUserId the identifier of the target user
	 * @param action the action of the permission to check
	 * @return true when a permission was found for this user, false otherwise
	 */
  public boolean checkUserPermission(int userId, int targetUserId, int action) {
		return checkUserPermission(em.find(User.class, userId), em.find(User.class, targetUserId), action);
	}

	/**
	 * Checks if the specified managed User entity has a user permission for the
	 * specified managed target User entity and action. A user has permission when
	 * a user permission for the specified target user and action exists for the
	 * user or for one of the groups where the user is a member of.
	 *
	 * @param user the managed User entity
	 * @param targetUser the managed target User entity
	 * @param action the action of the permission to check
	 * @return true when a user permission was found for this user, false otherwise
	 */
  public boolean checkUserPermission(User user, User targetUser, int action) {
    if (user == null) {
      return false;
    }

    if (getUserPermissionByUser(user, targetUser, action) != null) {
      return true;
    }

		List<Group> groups = groupBean.getByMember(user.getAppKey(), user, null);
    for (Group group : groups) {
      if (getUserPermissionByGroup(group, targetUser, action) != null) {
        return true;
      }
    }

    return false;
  }

	/**
	 * Find a user permission of a managed User entity for a specified managed
	 * target User entity and action.
	 *
	 * @param user the managed User entity
	 * @param targetUser the managed target User entity
	 * @param action the action of the permission
	 * @return A user permission of the user for the specified target user and action
	 */
  private UserPermission getUserPermissionByUser(User user, User targetUser, int action) {
    Query query = em.createNamedQuery("UserPermission.findByUserAndTargetAndAction");
    query.setParameter("user", user);
    query.setParameter("target", targetUser);
    query.setParameter("action", action);
    List<UserPermission> userPermissionsByUser = query.getResultList();
		return userPermissionsByUser.isEmpty() ? null : userPermissionsByUser.get(0);
	}

	/**
	 * Find a user permission of a managed Group entity that is of the specified
	 * type and action.
	 *
	 * @param group the managed Group entity
	 * @param targetUser the managed target User entity
	 * @param action the action of the permission
	 * @return A user permission of the group for the specified target user and action
	 */
  private UserPermission getUserPermissionByGroup(Group group, User targetUser, int action) {
    Query q = em.createNamedQuery("UserPermission.findByGroupAndTargetAndAction");
    q.setParameter("group", group);
    q.setParameter("target", targetUser);
    q.setParameter("action", action);
    List<UserPermission> userPermissionsByGroup = q.getResultList();
		return userPermissionsByGroup.isEmpty() ? null : userPermissionsByGroup.get(0);
	}

	/**
	 * Find all user permissions of a target user.
	 *
	 * @param targetUserId the identifier of the target user
	 * @return All user permissions of the target user
	 */
  public List<UserPermission> getUserPermissionsByTarget(int targetUserId) {
		return getUserPermissionsByTarget(em.find(User.class, targetUserId));
  }

	/**
	 * Find all user permissions of a managed target User entity.
	 *
	 * @param targetUser the managed target User entity
	 * @return All user permissions of the target user
	 */
  public List<UserPermission> getUserPermissionsByTarget(User targetUser) {
    Query q = em.createNamedQuery("UserPermission.findByTarget");
    q.setParameter("target", targetUser);
    return q.getResultList();
  }

	/**
	 * Find all user permissions of a user.
	 *
	 * @param userId the identifier of the user
	 * @return All user permissions of the user
	 */
  public List<UserPermission> getUserPermissionsByUser(int userId) {
		return getUserPermissionsByUser(em.find(User.class, userId));
  }

	/**
	 * Find all user permissions of a managed User entity.
	 *
	 * @param user the managed User entity
	 * @return All user permissions of the user
	 */
  public List<UserPermission> getUserPermissionsByUser(User user) {
    Query q = em.createNamedQuery("UserPermission.findByUser");
    q.setParameter("user", user);
    return q.getResultList();
  }

	/**
	 * Find all user permissions of a group.
	 *
	 * @param groupId the identifier of the group
	 * @return All user permissions of the group
	 */
  public List<UserPermission> getUserPermissionsByGroup(int groupId) {
		return getUserPermissionsByGroup(em.find(Group.class, groupId));
  }

	/**
	 * Find all user permissions of a managed Group entity.
	 *
	 * @param group the managed Group entity
	 * @return All user permissions of the group
	 */
  public List<UserPermission> getUserPermissionsByGroup(Group group) {
    Query q = em.createNamedQuery("UserPermission.findByGroup");
    q.setParameter("group", group);
    return q.getResultList();
  }

	/**
	 * Find all user permissions of a user for a certain target user.
	 *
	 * @param userId the identifier of the user
	 * @param targetUserId the identifier of the target user
	 * @return All user permissions of the user for the target user
	 */
  public List<UserPermission> getUserPermissionsByUser(int userId, int targetUserId) {
		return getUserPermissionsByUser(em.find(User.class, userId), em.find(User.class, targetUserId));
	}

	/**
	 * Find all user permissions of a managed User entity for a certain managed
	 * target User entity.
	 *
	 * @param user the managed User entity
	 * @param targetUser the managed target User entity
	 * @return All user permissions of the user for the target user
	 */
	public List<UserPermission> getUserPermissionsByUser(User user, User targetUser) {
    Query q = em.createNamedQuery("UserPermission.findByUserAndTarget");
    q.setParameter("user", user);
    q.setParameter("target", targetUser);
    List<UserPermission> result = q.getResultList();
    return result;
  }

	/**
	 * Find all user permissions of a group for a certain target user.
	 *
	 * @param groupId the identifier of the group
	 * @param targetUserId the identifier of the target user
	 * @return All user permissions of the group for the target user
	 */
	public List<UserPermission> getUserPermissionsByGroup(int groupId, int targetUserId) {
		return getUserPermissionsByGroup(em.find(Group.class, groupId), em.find(User.class, targetUserId));
	}

	/**
	 * Find all user permissions of a managed Group entity for a certain managed
	 * target User entity.
	 *
	 * @param group the managed Group entity
	 * @param targetUser the managed target User entity
	 * @return All user permissions of the group for the target user
	 */
  public List<UserPermission> getUserPermissionsByGroup(Group group, User targetUser) {
    Query q = em.createNamedQuery("UserPermission.findByGroupAndTarget");
    q.setParameter("group", group);
    q.setParameter("target", targetUser);
    List<UserPermission> result = q.getResultList();
    return result;
  }
}
