/*
 * Copyright (c) 2009-2011 LodgON  (http://www.lodgon.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package com.lodgon.dali.core.ejb;

import com.lodgon.dali.core.Constants;
import java.security.MessageDigest;
import java.util.Properties;
import java.util.logging.Level;

/**
 * Helper class for hashing user passwords.
 *
 * The default hashing algorithm that is used is SHA-256. This can be overridden
 * by including a project.properties file on the root classpath with the property
 * <code>dalicore_passwordhash_mechanism</code>. The value of this property
 * must be a hashing algorithm that is supported by {@link java.security.MessageDigest#getInstance(java.lang.String)}
 *
 * @see com.lodgon.dali.core.ejb.UserBean#setPassword(com.lodgon.dali.core.entity.User, java.lang.String)
 * @author erwin
 */
public class HashUtil {
	private static String mechanism = "SHA-256";

	static {
		try {
			Properties p = new Properties();
			p.load(HashUtil.class.getClassLoader().getResourceAsStream("project.properties"));
			mechanism = p.getProperty("dalicore_passwordhash_mechanism", "SHA-256");
		} catch (Exception ex) {
			Constants.LOGGER.log(Level.WARNING, "Exception while determining password hashing mechanism. Using default mechanism: SHA-256");
		}
		Constants.LOGGER.log(Level.INFO, "DaliCore configured to use password hashing mechanism {0}", HashUtil.mechanism);
	}

	/**
	 * Generates a message digest from the provided String. The algorithm that will
	 * be used is read from the property <code>dalicore_passwordhash_mechanism</code>
	 * in a project.properties file in the root classpath. The default algorithm
	 * is SHA-256.
	 *
	 * @param input a String to compute the message digest for.
	 * @return a message digest computed from the input.
	 */
	public static String getHash(String input) {
		try {
			MessageDigest md = MessageDigest.getInstance(HashUtil.mechanism);
			md.update(input.getBytes("UTF-8"));
			return toHexString(md.digest());
		} catch (Exception ex) {
			Constants.LOGGER.log(Level.SEVERE, "Exception while hashing input with mechanism " + HashUtil.mechanism, ex);
			return null;
		}
	}

	private static String toHexString(byte[] input) {
		StringBuilder hexString = new StringBuilder();
		for (int i = 0; i < input.length; i++) {
			String hex = Integer.toHexString(0xFF & input[i]);
			if (hex.length() == 1) {
				hexString.append("0");
			}
			hexString.append(hex);
		}
		return hexString.toString();
	}
}
