/*
 * Copyright (c) 2009-2011 LodgON  (http://www.lodgon.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package com.lodgon.dali.core.entity;

import java.io.Serializable;
import javax.persistence.Embeddable;

/**
 *
 * Class defining the different statuses a user can have.
 *
 * This is a regular class that can be extended by external code.
 * Currently, three default statuses exist:
 * <ul>
 * <li>ACTIVE: the user is a normal user
 * <li>BLOCKED: the user appears in search results, but validation of his/her
 * password will fail.
 * <li>HIDDEN: the user will not appear in search results.
 * </ul>
 *
 * @author joeri
 */
@Embeddable
public class Status implements Serializable {
	/**
	 * Users with status ACTIVE will appear in all search results and it will be
	 * possible to validate them. This is the default status for new users when
	 * their status was not explicitely set during creation.
	 *
	 * @see com.lodgon.dali.core.ejb.UserBean#validateEmailAndPassword(java.lang.String, java.lang.String)
	 * @see com.lodgon.dali.core.ejb.UserBean#validateScreenNameAndPassword(java.lang.String, java.lang.String)
	 * @see com.lodgon.dali.core.ejb.UserBean#validateUidAndPassword(java.lang.String, java.lang.String)
	 */
  public static final Status ACTIVE = new Status("ACTIVE");

	/**
	 * Users with status BLOCKED will appear in all search results, but trying to
	 * validate them will always fail.
	 *
	 * @see com.lodgon.dali.core.ejb.UserBean#validateEmailAndPassword(java.lang.String, java.lang.String)
	 * @see com.lodgon.dali.core.ejb.UserBean#validateScreenNameAndPassword(java.lang.String, java.lang.String)
	 * @see com.lodgon.dali.core.ejb.UserBean#validateUidAndPassword(java.lang.String, java.lang.String)
	 */
  public static final Status BLOCKED = new Status("BLOCKED");

	/**
	 * Users with status HIDDEN will not appear in search results by default, but
	 * it will be possible to validate the user. Note that it is possible to
	 * search for HIDDEN users by specifying the statuses to search for in the
	 * UserSearchParams.
	 *
	 * @see com.lodgon.dali.core.ejb.UserBean#search(com.lodgon.dali.core.entity.search.UserSearchParams)
	 */
  public static final Status HIDDEN = new Status("HIDDEN");

  private String status;

  public Status() {
  }

  public Status(String status) {
    this.status = status;
  }

  public String getStatus() {
    return status;
  }

  public void setStatus(String status) {
    this.status = status;
  }

  @Override
  public boolean equals(Object obj) {
    if (obj == null) {
      return false;
    }
    if (getClass() != obj.getClass()) {
      return false;
    }
    final Status other = (Status) obj;
    if ((this.status == null) ? (other.status != null) : !this.status.equals(other.status)) {
      return false;
    }
    return true;
  }

  @Override
  public int hashCode() {
    int hash = 7;
    hash = 79 * hash + (this.status != null ? this.status.hashCode() : 0);
    return hash;
  }

  @Override
  public String toString() {
    return "Status{" + "status=" + status + '}';
  }
    
}
