/*
 * Copyright (c) 2009-2011 LodgON  (http://www.lodgon.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package com.lodgon.dali.core.entity;

import java.io.Serializable;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.ManyToOne;
import javax.persistence.NamedQueries;
import javax.persistence.NamedQuery;
import javax.persistence.PostLoad;
import javax.persistence.PostPersist;
import javax.persistence.PostUpdate;
import javax.persistence.Transient;
import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlRootElement;

/**
 * The type permission is the most general of the permissions. It's a permission
 * that doesn't apply directly to one of the DaliCore entities. A type
 * permission can be granted to a user or group to provide a general permission
 * that is defined by a type and an action. An example can be a permission
 * to allow someone to remove (action) groups (type).
 *
 * @author erwin
 */
@Entity
@NamedQueries({
  @NamedQuery(
    name = "TypePermission.findByGroupAndTypeAndAction",
    query = "SELECT tp FROM TypePermission tp WHERE tp.group = :group AND tp.type = :type AND tp.action = :action"
  ),
  @NamedQuery(
    name = "TypePermission.findByUserAndTypeAndAction",
    query = "SELECT tp FROM TypePermission tp WHERE tp.user = :user AND tp.type = :type AND tp.action = :action"
  ),
  @NamedQuery(
    name = "TypePermission.findByType",
    query = "SELECT tp FROM TypePermission tp WHERE tp.type = :type ORDER BY tp.id, tp.action"
  ),
  @NamedQuery(
    name = "TypePermission.findByGroup",
    query = "SELECT tp FROM TypePermission tp WHERE tp.group = :group ORDER BY tp.id, tp.action"
  ),
  @NamedQuery(
    name = "TypePermission.findByGroupAndType",
    query = "SELECT tp FROM TypePermission tp WHERE tp.group = :group AND tp.type = :type ORDER BY tp.id, tp.action"
  ),
  @NamedQuery(
    name = "TypePermission.findByUser",
    query = "SELECT tp FROM TypePermission tp WHERE tp.user = :user ORDER BY tp.id, tp.action"
  ),
  @NamedQuery(
    name = "TypePermission.findByUserAndType",
    query = "SELECT tp FROM TypePermission tp WHERE tp.user = :user AND tp.type = :type ORDER BY tp.id, tp.action"
  )
})
@XmlRootElement
@XmlAccessorType(XmlAccessType.NONE)
public class TypePermission implements Serializable {
  private static final long serialVersionUID = 1L;
  @Id
  @GeneratedValue(strategy = GenerationType.IDENTITY)
  private Integer id;
  @ManyToOne
  private User user;
  @ManyToOne
  private Group group;
  private int type;
  @Column(name = "PERMISSIONACTION")
  private int action;

  @Transient
  private Authorizable authorizable;

	/**
	 * The action associated with this type permission.
	 *
	 * @return the action of the type permission
	 */
  public int getAction() {
    return action;
  }

	/**
	 * Sets the action associated with this type permission.
	 *
	 * @param action the action of the type permission
	 */
  public void setAction(int action) {
    this.action = action;
  }

	/**
	 * The group that was granted this type permission. If the permission was 
	 * instead granted to a user, then the group will be <code>null</code>.
	 *
	 * @return the group that was granted the type permission
	 */
  public Group getGroup() {
    return group;
  }

	/**
	 * Grants the type permission to the specified group.
	 *
	 * @param group the group that this type permission will be granted to
	 */
  public void setGroup(Group group) {
    this.group = group;
  }

	/**
	 * The unique identifier automatically generated by the persistence provider.
	 *
	 * @return unique identifier of the type permission
	 */
  public Integer getId() {
    return id;
  }

	/**
	 * Sets the unique identifier for the type permission. Note that this method
	 * should never be called by developers as it will be automatically set by the
	 * underlying persistence provider.
	 *
	 * @param id the unique identifier
	 */
  public void setId(Integer id) {
    this.id = id;
  }

	/**
	 * The type that is associated with this type permission.
	 *
	 * @return the type of the type permission
	 */
  public int getType() {
    return type;
  }

	/**
	 * Sets the type associated with this type permission.
	 *
	 * @param type the type of the type permission
	 */
  public void setType(int type) {
    this.type = type;
  }

	/**
	 * The user that was granted this type permission. If the permission was 
	 * instead granted to a group, then the user will be <code>null</code>.
	 *
	 * @return the user that was granted the type permission
	 */
  public User getUser() {
    return user;
  }

	/**
	 * Grants the type permission to the specified user.
	 *
	 * @param user the user that this type permission will be granted to
	 */
  public void setUser(User user) {
    this.user = user;
  }

	/**
	 * An authorizable that represents either the user or group that this
	 * type permission was granted to.
	 *
	 * @return an authorizable representing the entity this type permission was granted to
	 */
  public Authorizable getAuthorizable() {
    return authorizable;
  }

  @PostLoad
  @PostUpdate
  @PostPersist
  public void postLoad() {
    authorizable = new Authorizable();
    if (group != null) {
      authorizable.setId(group.getId());
			authorizable.setUid(group.getUid());
      authorizable.setType("group");
    }
    if (user != null) {
      authorizable.setId(user.getId());
			authorizable.setUid(user.getUid());
      authorizable.setType("user");
    }
  }
}
