/*
 * Copyright (c) 2009-2011 LodgON  (http://www.lodgon.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package com.lodgon.dali.core.entity;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import javax.persistence.CascadeType;
import javax.persistence.Embedded;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.NamedQueries;
import javax.persistence.NamedQuery;
import javax.persistence.OneToMany;
import javax.persistence.PrePersist;
import javax.persistence.Table;
import javax.persistence.Transient;
import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlRootElement;

/**
 * An instance of User contains the core fields of a typical user. A user has
 * the following relations with the other DaliCore entities:
 * 
 * <ul>
 *   <li>author: a user can be specified as the author of a Content or a Group</li>
 *   <li>member: a user can be a member of a Group</li>
 *   <li>authorizable: a user can be used in all permissions as the authorizable</li>
 *   <li>permission target: a user can be the target of a UserPermission</li>
 * </ul>
 *
 * @author joeri
 */
@Entity
@Table(name = "DALIUSER")
@NamedQueries({
  @NamedQuery(
    name = "User.findByAppKey",
    query = "SELECT u FROM User u WHERE u.appKey = :appKey"
  ),
  @NamedQuery(
    name = "User.findByUid",
    query = "SELECT u FROM User u WHERE u.uid = :uid AND u.appKey = :appKey AND u.status <> :hidden"
  ),
  @NamedQuery(
    name = "User.findByEmail",
    query = "SELECT u FROM User u WHERE u.email = :email AND u.appKey = :appKey AND u.status <> :hidden"
  ),
  @NamedQuery(
    name = "User.findByScreenName",
    query = "SELECT u FROM User u WHERE u.screenName = :screenName AND u.appKey = :appKey AND u.status <> :hidden"
  ),
  @NamedQuery(
    name = "User.validateByUid",
    query = "SELECT u FROM User u WHERE u.uid = :uid AND u.appKey = :appKey AND u.status <> :blocked"
  ),
  @NamedQuery(
    name = "User.validateByEmail",
    query = "SELECT u FROM User u WHERE u.email = :email AND u.appKey = :appKey AND u.status <> :blocked"
  ),
  @NamedQuery(
    name = "User.validateByScreenName",
    query = "SELECT u FROM User u WHERE u.screenName = :screenName AND u.appKey = :appKey AND u.status <> :blocked"
  ),
  @NamedQuery(
    name = "User.count",
    query = "SELECT COUNT(u) FROM User u WHERE u.appKey = :appKey AND u.status <> :hidden"
	)
})
@XmlRootElement
@XmlAccessorType(XmlAccessType.NONE)
public class User implements Serializable {
  private static final long serialVersionUID = 1L;
  @Id
  @GeneratedValue(strategy = GenerationType.IDENTITY)
  private Integer id;

	private int type;
  private String screenName;
  private String firstName;
  private String lastName;
  private String email;
  private String password;
  private String depiction;
  private long creationDate;
  private long lastLoginDate;
  private String uid;

  @OneToMany(mappedBy= "user", orphanRemoval = true, cascade=CascadeType.REMOVE, fetch=FetchType.LAZY)
  private List<TypePermission> typePermissions;

  @OneToMany(mappedBy= "user", orphanRemoval = true, cascade=CascadeType.REMOVE, fetch=FetchType.LAZY)
  private List<ContentPermission> contentPermissions;

  @OneToMany(mappedBy= "user", orphanRemoval = true, cascade=CascadeType.REMOVE, fetch=FetchType.LAZY)
  private List<GroupPermission> groupPermissions;

	@OneToMany(mappedBy= "target", orphanRemoval = true, cascade=CascadeType.REMOVE, fetch=FetchType.LAZY)
  private List<UserPermission> targetUserPermissions;

	@OneToMany(mappedBy= "user", orphanRemoval = true, cascade=CascadeType.REMOVE, fetch=FetchType.LAZY)
  private List<UserPermission> userPermissions;

	@OneToMany(mappedBy = "user", orphanRemoval = true, cascade=CascadeType.REMOVE, fetch=FetchType.LAZY)
	private List<Member> memberships;

	@OneToMany(mappedBy = "user", orphanRemoval = true, cascade = CascadeType.ALL)
  private List<Field> fields;

  @Transient
  private Map<String,Field> fieldmap;

	private String appKey;

  @Embedded
  private Status status;

	/**
	 * The unique identifier automatically generated by the persistence provider.
	 *
	 * @return unique identifier of the user
	 */
  public Integer getId() {
    return id;
  }

	/**
	 * Sets the unique identifier for the user. Note that this method should never
	 * be called by developers as it will be automatically set by the underlying
	 * persistence provider.
	 *
	 * @param id the unique identifier
	 */
  public void setId(Integer id) {
    this.id = id;
  }

	/**
	 * The time in milliseconds when the user was created.
	 *
	 * @return the time in milliseconds since midnight, January 1, 1970 UTC
	 */
  public long getCreationDate() {
    return creationDate;
  }

	/**
	 * Sets the time when the user was created. Note that the creation date will
	 * be automatically set if it was not provided when persisting the user.
	 *
	 * @param creationDate the time in milliseconds when the user was created
	 */
  public void setCreationDate(long creationDate) {
    this.creationDate = creationDate;
  }

	/**
	 * The URL to the profile image of the user.
	 *
	 * @return the URL of the user's profile image
	 */
  public String getDepiction() {
    return depiction;
  }

	/**
	 * Defines the URL location of the profile image to use for this user.
	 *
	 * @param depiction the URL of the profile image
	 */
  public void setDepiction(String depiction) {
    this.depiction = depiction;
  }

	/**
	 * The e-mail address of the user.
	 *
	 * @return the user's e-mail address
	 */
  public String getEmail() {
    return email;
  }

	/**
	 * Defines the e-mail address for this user. When you user user authentication
	 * in your project, you can choose the e-mail address as the unique identifier
	 * for your users.
	 *
	 * @param email the e-mail address of the user
	 */
  public void setEmail(String email) {
    this.email = email;
  }

	/**
	 * The hashed password of the user.
	 *
	 * @return the user's hashed password
	 */
  public String getPassword() {
    return password;
  }

	/**
	 * Sets the hashed password for this user. Note that you should not call this
	 * method directly but rather use {@link com.lodgon.dali.core.ejb.UserBean#setPassword(int, java.lang.String) UserBean.setPassword()}
	 * instead.
	 *
	 * @param password the hashed password for the user
	 */
  public void setPassword(String password) {
    this.password = password;
  }

	/**
	 * The first name the user.
	 *
	 * @return the user's first name
	 */
  public String getFirstName() {
    return firstName;
  }

	/**
	 * Defines the first name for the user.
	 *
	 * @param firstName the first name of the user
	 */
  public void setFirstName(String firstName) {
    this.firstName = firstName;
  }

	/**
	 * The time in milliseconds when the user was last validated.
	 *
	 * @return the time in milliseconds since midnight, January 1, 1970 UTC
	 */
  public long getLastLoginDate() {
    return lastLoginDate;
  }

	/**
	 * Sets the last time a user was validated. Using any of the validation
	 * methods on UserBean will automatically update this field when the
	 * validation for the user was successful.
	 *
	 * @param lastLoginDate the time in milliseconds when the user was last validated
	 * @see com.lodgon.dali.core.ejb.UserBean#validateEmailAndPassword(java.lang.String, java.lang.String)
	 * @see com.lodgon.dali.core.ejb.UserBean#validateScreenNameAndPassword(java.lang.String, java.lang.String)
	 * @see com.lodgon.dali.core.ejb.UserBean#validateUidAndPassword(java.lang.String, java.lang.String)
	 */
  public void setLastLoginDate(long lastLoginDate) {
    this.lastLoginDate = lastLoginDate;
  }

	/**
	 * The last name of the user.
	 *
	 * @return the user's last name
	 */
  public String getLastName() {
    return lastName;
  }

	/**
	 * Defines the last name for the user.
	 *
	 * @param lastName the last name of the user
	 */
  public void setLastName(String lastName) {
    this.lastName = lastName;
  }

  /**
   * The display name of the user. If either the first name or last name is
	 * set, then the display name will look like <code>firstname lastname</code>.
	 * In case both the first name and last name are <code>null</code>, the
	 * <code>screenName</code> will be used instead.
	 *
   * @return the display name of the user
   */
  public String getDisplayName() {
    StringBuilder b = new StringBuilder();
    if (firstName != null && firstName.length() > 0) {
      b.append(firstName);
    }
    if (lastName != null && lastName.length() > 0) {
      if (b.length() > 0) {
        b.append(" ");
      }
      b.append(lastName);
    }
    if (b.length() == 0) {
      return screenName;
    }
    return b.toString();
  }

	/**
	 * The current status of the user.
	 *
	 * @return the current status of the user
	 */
  public Status getStatus() {
    return status;
  }

	/**
	 * Sets the current status for the user. Setting the status has an impact
	 * on the logic of some of the methods in the
	 * {@link com.lodgon.dali.core.ejb.UserBean UserBean}.
	 *
	 * @param status the new status for the user
	 */
  public void setStatus(Status status) {
    this.status = status;
  }

	/**
	 * The screen name of the user.
	 *
	 * @return the user's screen name
	 */
  public String getScreenName() {
    return screenName;
  }

	/**
	 * Defines the screen name for this user. When you user user authentication
	 * in your project, you can choose the screen name as the unique identifier
	 * for your users.
	 *
	 * @param screenName the screen name of the user
	 */
  public void setScreenName(String screenName) {
    this.screenName = screenName;
  }

	/**
	 * The type of the user.
	 *
	 * @return the user's type
	 */
	public int getType() {
		return type;
	}

	/**
	 * Sets the type of the user. A type can be used to categorize a list of
	 * users that define the same logical entity.
	 *
	 * @param type the type of the user
	 */
	public void setType(int type) {
		this.type = type;
	}

  /**
	 * The unique identifier for this user.
	 *
   * @return the user's unique identifier
   */
  public String getUid() {
    return uid;
  }

  /**
	 * A unique identifier that can optionally be set by an application. In
	 * contrast to the {@link #setId(java.lang.Integer) id}, this value can be
	 * custom provided by an application. The only requirement is that the
	 * value needs to be unique over all users. When the <code>uid</code> is not
	 * provided when persisting a user, then it will be set automatically to a
	 * {@link java.util.UUID#randomUUID() UUID}.
	 *
   * @param uid the unique String identifier of the user
   */
  public void setUid(String uid) {
    this.uid = uid;
  }

	/**
	 * A list of memberships for the user.
	 *
	 * @return all the memberships for the user
	 */
	public List<Member> getMemberships() {
		return memberships;
	}

	/**
	 * Sets the memberships for the user. Instead of using this method,
	 * controlling the memberships of a user should be done by using the
	 * {@link com.lodgon.dali.core.ejb.GroupBean#join(int, int) join} and
	 * {@link com.lodgon.dali.core.ejb.GroupBean#leave(int, int) leave} methods.
	 *
	 * @param memberships the memberships for the user
	 */
	public void setMemberships(List<Member> memberships) {
		this.memberships = memberships;
	}

	/**
	 * The application key that is associated with this user.
	 *
	 * @return the user's application key
	 */
  public String getAppKey() {
    return appKey;
  }

	/**
	 * Sets the application key associated with the user. The application key
	 * defines the application scope of the user object. By setting an application
	 * key, this user can only interact with other entities within the same
	 * application scope.
	 *
	 * @param appKey the application key to set
	 */
  public void setAppKey(String appKey) {
    this.appKey = appKey;
  }

	/**
	 * A list of content permissions that were granted to this user.
	 *
	 * @return all content permissions granted to the user
	 */
  public List<ContentPermission> getContentPermissions() {
    return contentPermissions;
  }

	/**
	 * Sets the list of content permissions that were granted to this user.
	 *
	 * @param contentPermissions a list of content permissions granted to this user.
	 */
  public void setContentPermissions(List<ContentPermission> contentPermissions) {
    this.contentPermissions = contentPermissions;
  }

	/**
	 * A list of group permissions that were granted to this user.
	 *
	 * @return all group permissions granted to the user
	 */
  public List<GroupPermission> getGroupPermissions() {
    return groupPermissions;
  }

	/**
	 * Sets the list of group permissions that were granted to this user.
	 *
	 * @param groupPermissions a list of group permissions granted to this user.
	 */
  public void setGroupPermissions(List<GroupPermission> groupPermissions) {
    this.groupPermissions = groupPermissions;
  }

	/**
	 * A list of user permissions that were set on this user.
	 *
	 * @return all user permissions set on this user
	 */
  public List<UserPermission> getTargetUserPermissions() {
    return targetUserPermissions;
  }

	/**
	 * Sets the list of user permissions of which this user is the target.
	 *
	 * @param targetUserPermissions a list of user permissions set on this user
	 */
  public void setTargetUserPermissions(List<UserPermission> targetUserPermissions) {
    this.targetUserPermissions = targetUserPermissions;
  }

	/**
	 * A list of user permissions that were granted to this user.
	 *
	 * @return all user permissions granted to the user
	 */
  public List<UserPermission> getUserPermissions() {
    return userPermissions;
  }

	/**
	 * Sets the list of user permissions that were granted to this user.
	 *
	 * @param userPermissions a list of user permissions granted to this user.
	 */
  public void setUserPermissions(List<UserPermission> userPermissions) {
    this.userPermissions = userPermissions;
  }

	/**
	 * A list of type permissions that were granted to this user.
	 *
	 * @return all type permissions granted to the user
	 */
  public List<TypePermission> getTypePermissions() {
    return typePermissions;
  }

	/**
	 * Sets the list of type permissions that were granted to this user.
	 *
	 * @param typePermissions a list of type permissions granted to this user.
	 */
  public void setTypePermissions(List<TypePermission> typePermissions) {
    this.typePermissions = typePermissions;
  }


	/**
	 * A list of custom fields defined for this user.
	 *
	 * @return the custom fields of the user
	 */
  public List<Field> getFields() {
    return fields;
  }

	/**
	 * Sets the list of custom fields to be defined for this user.
	 *
	 * @param fields the complete list of custom fields
	 */
  public void setFields(List<Field> fields) {
    this.fields = fields;
  }

	/**
	 * Adds a new field to the list of custom user fields. This won't override
	 * any fields that already might have been set.
	 *
	 * @param field the new field to be added
	 */
  public void addField(Field field) {
    if (fields == null) {
      fields = new LinkedList();
    }
    fields.add(field);
    if (fieldmap != null) {
      fieldmap.put(field.getName(), field);
    }
  }

	/**
	 * Updates the field that has the same name as the specified name. Note that in
	 * case there are fields that share the same name, it is not predictable which
	 * field will be updated.
	 *
	 * @param name the name of the field to be updated
	 * @param newValue the new value to be set on the field
	 * @return the field with the updated value or <code>null</code> if no field
	 * was found with the specified name
	 */
  public Field updateFieldValue(String name, String newValue) {
    Field field = getField().get(name);
    if (field != null) {
      field.setValue(newValue);
    }
    return field;
  }

	/**
	 * Removes the field with the specified name. Note that in case there are
	 * fields that share the same name, it is not predictable which field will be
	 * removed.
	 *
	 * @param name the name of the field to be removed
	 */
  public void removeField(String name) {
    if (fields != null) {
      for (Iterator<Field> it = fields.iterator();it.hasNext();) {
        Field field = it.next();
        if (field.getName().equals(name)) {
          it.remove();
          return;
        }
      }
      fieldmap = null;
    }
  }

	/**
	 * Returns a map where the names of each field are mapped with their
	 * corresponding field object. Note that in case there are fields that share
	 * the same name, only one of them will be made available in the map.
	 *
	 * @return a mapping between the name of a field and the field itself
	 */
  public Map<String,Field> getField() {
    if (fieldmap == null) {
      fieldmap = new HashMap();
      if (fields != null) {
        for (Field f : fields) {
          fieldmap.put(f.getName(), f);
        }
      }
    }
    return fieldmap;
  }

	@Override
	public boolean equals(Object obj) {
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		final User other = (User) obj;
		if (this.id != other.id && (this.id == null || !this.id.equals(other.id))) {
			return false;
		}
		if ((this.uid == null) ? (other.uid != null) : !this.uid.equals(other.uid)) {
			return false;
		}
		return true;
	}

	@Override
	public int hashCode() {
		int hash = 7;
		hash = 79 * hash + (this.id != null ? this.id.hashCode() : 0);
		return hash;
	}

  @Override
  public String toString() {
    return "com.lodgon.dali.core.entity.User[id=" + id + "]";
  }

  @PrePersist
  public void prePersist() {
    if (creationDate == 0) {
      this.creationDate = System.currentTimeMillis();
    }
  }

  protected void copyFrom(User src) {
    setAppKey(appKey);
    setContentPermissions(src.getContentPermissions());
    setCreationDate(src.getCreationDate());
    setEmail(src.getEmail());
    setFields(src.getFields());
    setFirstName(src.getFirstName());
    setId(src.getId());
    setLastLoginDate(src.getLastLoginDate());
    setLastName(src.getLastName());
    setPassword(src.getPassword());
    setTypePermissions(src.getTypePermissions());
    setScreenName(src.getScreenName());
  }
}
