/*
 * Copyright (c) 2009-2011 LodgON  (http://www.lodgon.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package com.lodgon.dali.core.entity.search;

import com.lodgon.dali.core.entity.Field;

/**
 * The base class for building instances of SearchParams using the builder
 * pattern.
 *
 * @author joeri
 */
public abstract class SearchParamsBuilder<T extends SearchParamsBuilder, U extends SearchParams> {
	protected U params;

	/**
	 * Creates a builder to construct an instance of {@link com.lodgon.dali.core.entity.search.ContentSearchParams}.
	 *
	 * @return a builder to construct ContentSearchParams
	 */
	public static ContentSearchParamsBuilder content() {
		return new ContentSearchParamsBuilder();
	}

	/**
	 * Creates a builder to construct an instance of {@link com.lodgon.dali.core.entity.search.GroupSearchParams}.
	 *
	 * @return a builder to construct GroupSearchParams
	 */
	public static GroupSearchParamsBuilder group() {
		return new GroupSearchParamsBuilder();
	}

	/**
	 * Creates a builder to construct an instance of {@link com.lodgon.dali.core.entity.search.UserSearchParams}.
	 *
	 * @return a builder to construct UserSearchParams
	 */
	public static UserSearchParamsBuilder user() {
		return new UserSearchParamsBuilder();
	}

	/**
	 * Sets the {@link com.lodgon.dali.core.entity.search.SearchParams#setQuantification(com.lodgon.dali.core.entity.search.SearchParams.Quantification) quantification}
	 * property for the instance constructed by this builder to
	 * {@link com.lodgon.dali.core.entity.search.SearchParams.Quantification#ALL ALL}.
	 */
	public T all() {
		params.setQuantification(SearchParams.Quantification.ALL);
		return (T) this;
	}

	/**
	 * Sets the {@link com.lodgon.dali.core.entity.search.SearchParams#setQuantification(com.lodgon.dali.core.entity.search.SearchParams.Quantification) quantification}
	 * property for the instance constructed by this builder to
	 * {@link com.lodgon.dali.core.entity.search.SearchParams.Quantification#ANY ANY}.
	 */
	public T any() {
		params.setQuantification(SearchParams.Quantification.ANY);
		return (T) this;
	}

	/**
	 * Sets the {@link com.lodgon.dali.core.entity.search.SearchParams#setOrder(com.lodgon.dali.core.entity.search.SearchParams.Order) order}
	 * property for the instance constructed by this builder to
	 * {@link com.lodgon.dali.core.entity.search.SearchParams.Order#ASCENDING ASCENDING}.
	 */
	public T asc() {
		params.setOrder(SearchParams.Order.ASCENDING);
		return (T) this;
	}

	/**
	 * Sets the {@link com.lodgon.dali.core.entity.search.SearchParams#setDateFrom(java.lang.Long) dateFrom}
	 * property for the instance constructed by this builder.
	 */
	public T dateFrom(Long dateFrom) {
		params.setDateFrom(dateFrom);
		return (T) this;
	}

	/**
	 * Sets the {@link com.lodgon.dali.core.entity.search.SearchParams#setDateTo(java.lang.Long) dateTo}
	 * property for the instance constructed by this builder.
	 */
	public T dateTo(Long dateTo) {
		params.setDateTo(dateTo);
		return (T) this;
	}

	/**
	 * Sets the {@link com.lodgon.dali.core.entity.search.SearchParams#setOrder(com.lodgon.dali.core.entity.search.SearchParams.Order) order}
	 * property for the instance constructed by this builder to
	 * {@link com.lodgon.dali.core.entity.search.SearchParams.Order#DESCENDING DESCENDING}.
	 */
	public T desc() {
		params.setOrder(SearchParams.Order.DESCENDING);
		return (T) this;
	}

	/**
	 * Adds a value to the {@link com.lodgon.dali.core.entity.search.SearchParams#setFields(java.util.List) fields}
	 * property for the instance constructed by this builder.
	 */
	public T field(String name, String value) {
		params.getFields().add(new Field(name, value));
		return (T) this;
	}

	/**
	 * Sets the {@link com.lodgon.dali.core.entity.search.SearchParams#setOffset(java.lang.Integer) offset}
	 * property for the instance constructed by this builder.
	 */
	public T offset(Integer offset) {
		params.setOffset(offset);
		return (T) this;
	}

	/**
	 * Sets the {@link com.lodgon.dali.core.entity.search.SearchParams#setMax(java.lang.Integer) max}
	 * property for the instance constructed by this builder.
	 */
	public T max(Integer max) {
		params.setMax(max);
		return (T) this;
	}

	/**
	 * Sets the {@link com.lodgon.dali.core.entity.search.SearchParams#setSort(java.lang.String) sort}
	 * property for the instance constructed by this builder.
	 */
	public T sort(String sort) {
		params.setSort(sort);
		return (T) this;
	}

	/**
	 * Adds a value to the {@link com.lodgon.dali.core.entity.search.SearchParams#setTypes(java.util.Set) types}
	 * property for the instance constructed by this builder.
	 */
	public T type(Integer type) {
		params.getTypes().add(type);
		return (T) this;
	}

	/**
	 * Creates an instance of SearchParams based on the parameters that were set
	 * on this builder.
	 *
	 * @return an instance of SearchParams that can be used to search for entities
	 */
	public U create() {
		return params;
	}
}
