/*
 * Copyright (c) 2009-2011 LodgON  (http://www.lodgon.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package com.lodgon.dali.core.search;

import com.lodgon.dali.core.entity.Field;
import com.lodgon.dali.core.entity.Group;
import com.lodgon.dali.core.entity.Member;
import com.lodgon.dali.core.entity.User;
import com.lodgon.dali.core.entity.search.GroupSearchParams;
import com.lodgon.dali.core.entity.search.SearchParams;
import java.util.LinkedList;
import java.util.List;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.TypedQuery;
import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Expression;
import javax.persistence.criteria.Join;
import javax.persistence.criteria.JoinType;
import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;

/**
 * This is the default implementation of SearchProvider for Group entities.
 * When no other SearchProvider implementation was found through CDI injection,
 * then DaliCore will fall back to this implementation, which performs its
 * searches by building JPA queries with the Criteria API.
 *
 * @author Joeri
 */
public class DefaultGroupSearchProvider implements SearchProvider<Group, GroupSearchParams> {
	@PersistenceContext
  EntityManager em;

	@Override
	public Long count(String appKey, GroupSearchParams params) {
		// find out if an ID was provided of a non existing entity. when there is
		// one found, return an empty list
		if (params.getAuthorId() != null) {
			User author = em.find(User.class, params.getAuthorId());
			if (author != null) {
				params.setAuthor(author);
			} else {
				return Long.valueOf(0);
			}
		}
		if (params.getMemberId() != null) {
			User member = em.find(User.class, params.getMemberId());
			if (member != null) {
				params.setMember(member);
			} else {
				return Long.valueOf(0);
			}
		}
		if (params.getParentId() != null) {
			Group parent = em.find(Group.class, params.getParentId());
			if (parent != null) {
				params.setParent(parent);
			} else {
				return Long.valueOf(0);
			}
		}

		CriteriaBuilder builder = em.getCriteriaBuilder();
		CriteriaQuery<Long> query = builder.createQuery(Long.class);

		Root<Group> _group = query.from(Group.class);

		Expression where = createWhereExpression(appKey, builder, _group, params);

		query.select(builder.countDistinct(_group)).where(where);

		TypedQuery<Long> q = em.createQuery(query);
		return q.getSingleResult();
	}

	@Override
	public List<Group> search(String appKey, GroupSearchParams params) {
		// find out if an ID was provided of a non existing entity. when there is
		// one found, return an empty list
		if (params.getAuthorId() != null) {
			User author = em.find(User.class, params.getAuthorId());
			if (author != null) {
				params.setAuthor(author);
			} else {
				return new LinkedList<Group>();
			}
		}
		if (params.getMemberId() != null) {
			User member = em.find(User.class, params.getMemberId());
			if (member != null) {
				params.setMember(member);
			} else {
				return new LinkedList<Group>();
			}
		}
		if (params.getParentId() != null) {
			Group parent = em.find(Group.class, params.getParentId());
			if (parent != null) {
				params.setParent(parent);
			} else {
				return new LinkedList<Group>();
			}
		}

		CriteriaBuilder builder = em.getCriteriaBuilder();
		CriteriaQuery<Group> query = builder.createQuery(Group.class);

		Root<Group> _group = query.from(Group.class);

		Expression where = createWhereExpression(appKey, builder, _group, params);

		query.distinct(true).select(_group).where(where);

		Expression sort = _group.get("creationDate");
		if (params.getSort() != null
						&& (params.getSort().equals("name")
						|| params.getSort().equals("path")
						|| params.getSort().equals("type")
						|| params.getSort().equals("creationDate"))) {
			sort = _group.get(params.getSort());
		}

		if (params.getOrder() == null || params.getOrder() == SearchParams.Order.ASCENDING) {
			query.orderBy(builder.asc(sort), builder.asc(_group.get("id")));
		} else {
			query.orderBy(builder.desc(sort), builder.asc(_group.get("id")));
		}

		TypedQuery<Group> q = em.createQuery(query);
		if (params.getOffset() != null) {
			q.setFirstResult(params.getOffset());
		}
		if (params.getMax() != null) {
			q.setMaxResults(params.getMax());
		}
		return q.getResultList();
	}

	private Expression createWhereExpression(String appKey, CriteriaBuilder builder, Root<Group> _group, GroupSearchParams params) {
		SearchParams.Quantification quantor = params.getQuantification();

		Expression where = null;
		if (params.getDateFrom() != null) {
			where = addExpression(builder, where, builder.gt(_group.<Long>get("creationDate"), params.getDateFrom()), quantor);
		}
		if (params.getDateTo() != null) {
			where = addExpression(builder, where, builder.lt(_group.<Long>get("creationDate"), params.getDateTo()), quantor);
		}
		if (params.getNames() != null) {
			Expression whereNames = null;
			for (String name : params.getNames()) {
				whereNames = addExpression(builder, whereNames, builder.like(_group.<String>get("name"), name.replaceAll("\\*", "%")), SearchParams.Quantification.ANY);
			}
			if (whereNames != null) {
				where = addExpression(builder, where, whereNames, quantor);
			}
		}

		if (params.getAuthor() != null) {
			where = addExpression(builder, where, builder.equal(_group.<User>get("author"), params.getAuthor()), quantor);
		}

		if (params.getMember() != null) {
			where = addExpression(builder, where, builder.equal(_group.joinList("memberships", JoinType.LEFT).get("user"), params.getMember()), quantor);
		}

		if (params.getParent() != null) {
			if (params.getDepth() == null || params.getDepth() == 1) {
				where = addExpression(builder, where, builder.equal(_group.<User>get("parent"), params.getParent()), quantor);
			} else {
				Predicate pathPredicate = builder.like(_group.<String>get("path"), (params.getParent().getPath().isEmpty() ? "/" + params.getParent().getId() : params.getParent().getPath() + params.getParent().getId()) + "/%");
				if (params.getDepth() < 0) {
					where = addExpression(builder, where, pathPredicate, quantor);
				} else {
					where = addExpression(builder, where, builder.and(pathPredicate, builder.le(_group.<Integer>get("depth"), params.getParent().getDepth() + params.getDepth())), quantor);
				}
			}
		}

    if (params.getFields() != null && ! params.getFields().isEmpty()) {
      for (Field field : params.getFields()) {
	      Join<Group, Field> _fields = _group.joinList("fields", JoinType.LEFT);
        Predicate fieldsPredicate = builder.and(builder.equal(_fields.get("name"), field.getName()), builder.like(_fields.<String>get("value"), field.getValue().replaceAll("\\*", "%")));
        where = addExpression(builder, where, fieldsPredicate, quantor);
      }
    }

		if (params.getTypes() != null && ! params.getTypes().isEmpty()) {
			where = addExpression(builder, where, _group.get("type").in(params.getTypes()), SearchParams.Quantification.ALL);
		}

		return addExpression(builder, where, builder.equal(_group.get("appKey"), appKey != null ? appKey : ""), SearchParams.Quantification.ALL);
	}

	private Expression addExpression(CriteriaBuilder builder, Expression where, Expression expression, SearchParams.Quantification quantification) {
		if (where == null) {
			return expression;
		} else {
			if (quantification == SearchParams.Quantification.ANY) {
				return builder.or(where, expression);
			} else {
				return builder.and(where, expression);
			}
		}
	}
}
